package com.bonait.bnframework.business;

import android.app.Activity;
import android.content.Context;
import android.content.ContextWrapper;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import androidx.annotation.NonNull;

import com.bonait.bnframework.common.constant.ConfigName;
import com.bonait.bnframework.common.db.QueryDB;
import com.bonait.bnframework.common.db.mode.BPA_GOODSRECIPE;
import com.bonait.bnframework.common.db.mode.BPA_PLCADDRESS;
import com.bonait.bnframework.common.db.mode.BPA_SILOS;
import com.bonait.bnframework.common.helper.I.IRun;
import com.bonait.bnframework.common.helper.I.IRunT;
import com.bonait.bnframework.common.helper.I.IThread;
import com.bonait.bnframework.common.helper.I.IWriteCallBack;
import com.bonait.bnframework.common.helper.ThreadManager;
import com.bonait.bnframework.common.modbus.s7.SiemensHelper1;
import com.bonait.bnframework.common.utils.ToastUtils;
import com.qmuiteam.qmui.widget.dialog.QMUIDialog;
import com.qmuiteam.qmui.widget.dialog.QMUIDialogAction;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 执行配方
 */
public class ExecuteTheRecipe {

    //region 变量
    /**
     * 等待超时时间
     */
    public static int whileTime = 2;
    /**
     * 监听变量值
     */
    public static HashMap<String, Object> ListeningValue = new HashMap<String, Object>();
    /**
     * 外部界面
     */
    public static Context context = null;
    /**
     * 商品是否强制结束
     */
    public static boolean IsForcedEnd = false;
    /**
     * 是否启动
     */
    public static boolean IsStart = false;
    /**
     * 是否暂停
     */
    public static boolean IsPause = false;
    //endregion

    //region 强制结束

    /**
     * 设置强制结束
     */
    public static void SetForcedEnd() {
        ToastUtils.info("客官,小菠萝正在强制结束当前炒制菜品,请耐心等候!!!");
        IsForcedEnd = true;
    }

    /**
     * 停止强制结束
     */
    public static void StopForcedEnd() {
        IsForcedEnd = false;
    }
    //endregion

    //region 执行配方

    /**
     * 执行配方
     *
     * @param recipe
     * @return
     */
    public static boolean Execute(BPA_GOODSRECIPE recipe) {
        boolean status = false;
        try {
            //1.解析
            if (IsForcedEnd) {
                ExecuteTheRecipe.showlog("客官,小菠萝正在强制结束当前炒制菜品,请耐心等候!!!");
                return false;
            }
            ExecuteTheRecipe.showlog("开始执行:" + recipe.sort + "、" + recipe.processms);
            if (recipe.materialType == 0)//正常物料
            {
                //开始配料
                ExecuteMaterialIssuance(recipe.processvalue);
            } else if (recipe.materialType == 1)//工序模型
            {
                //执行工序,非物料工序
                ExecuteOperationSteps(recipe.processname, recipe.processvalue);
            }
        } catch (Exception ex) {
            ToastUtils.error("异常信息:" + ex.getMessage());
            status = false;
        } finally {
            return status;
        }
    }

    /**
     * 液体物料信息-配料
     *
     * @param text
     */
    public static void ExecuteMaterialIssuance(String text) {
        try {
            if (!text.isEmpty()) {
                //仓号 出料克数
                HashMap<BPA_SILOS, Integer> formulation = new HashMap<>();

                //region 获取仓号和值
                List<String> data = new ArrayList<>();
                if (text.contains("|")) {
                    String[] res = text.split("[|]");
                    for (int i = 0; i < res.length; i++) {
                        data.add(res[i]);
                    }
                } else {
                    data.add(text);
                }
                for (String item : data) {
                    if (!item.isEmpty() && item.contains(",")) {
                        String[] wl = item.split("[,]");
                        if (wl != null && wl.length == 2) {
                            String name = wl[0];
                            int val = (int) Math.round((Double.parseDouble(wl[1])));
                            List<BPA_SILOS> bpa_silos = QueryDB.GetSolisByMaterialName(name);
                            if (bpa_silos.size() > 0 && val > 0) {
                                BPA_SILOS silos = bpa_silos.get(0);
                                formulation.put(silos, val);
                            }
                        }
                    }
                }
                //endregion

                //写入PLC,直接出料，不用等待
                for (Map.Entry<BPA_SILOS, Integer> entry : formulation.entrySet()) {
                    BPA_SILOS silos = entry.getKey();
                    Integer value = entry.getValue();
                    Write_PLC_Material(silos, value);
                }
            }
        } catch (Exception ex) {
            ToastUtils.error("异常信息:" + ex.getMessage());
        }
    }

    /**
     * 执行工序步骤
     *
     * @param text
     */
    public static void ExecuteOperationSteps(String processname, String text) {
        try {
            if (!text.isEmpty() && !processname.isEmpty()) {
                //工序名称和值
                HashMap<String, String> formulation = new HashMap<>();
                //region 获取仓号和值
                List<String> data = new ArrayList<>();
                if (text.contains("|")) {
                    String[] res = text.split("[|]");
                    for (int i = 0; i < res.length; i++) {
                        data.add(res[i]);
                    }
                } else {
                    data.add(text);
                }
                for (String item : data) {
                    if (!item.isEmpty() && item.contains(",")) {
                        String[] wl = item.split("[,]");
                        if (wl != null && wl.length == 2) {
                            String name = wl[0];
                            String val = wl[1];
                            formulation.put(name, val);
                        }
                    }
                }
                //endregion
                //写入PLC
                if (formulation.size() > 0) {
                    switch (processname) {
                        case "主料":
                            Write_PLC_Ingredients(formulation);
                            break;
                        case "搅拌":
                            Write_PLC_Stir(formulation);
                            break;
                        case "火力设置":
                            Write_PLC_Heating(formulation);
                            break;
                        case "约束条件":
                            Write_PLC_ConstraintCondition(formulation);
                            break;

                    }
                }
            }
        } catch (Exception ex) {
            ToastUtils.error("异常信息:" + ex.getMessage());
        }
    }
    //endregion

    //region 物料PLC控制

    /**
     * 写入仓号需求值
     *
     * @param silos
     * @param val
     */
    private static void Write_PLC_Material(BPA_SILOS silos, int val) {
        try {
            ExecuteTheRecipe.showlog("准备写入PLC仓号需求:" + silos.num + "," + val);
            String name = "料仓" + silos.num + "出料";
            WritePLC(name, false); //复位料仓1号

            WritePLC("设定重量", val);
            WritePLC("设定提前量", 20);
            ExecuteTheRecipe.showlog("需求值写入成功！");

            //減去料仓数量
            int otherG = silos.silosmargin - (val / 10);
            QueryDB.UpdateYL(silos.id, otherG >= 0 ? otherG : 0);


            WritePLC(name, true); //开始出料

            //等待出料完成,false信号
            ThreadWhileFlase(name);
        } catch (Exception ex) {
            ToastUtils.error("异常信息:" + ex.getMessage());
        }
    }
    //endregion

    //region 写PLC工序

    /**
     * 写PLC主料
     *
     * @param data
     */
    private static void Write_PLC_Ingredients(HashMap<String, String> data) {
        try {
            String name = "";
            for (HashMap.Entry<String, String> entry : data.entrySet()) {
                String key = entry.getKey();
                String value = entry.getValue();
                if (key.contains("名称")) {
                    name = value;
                }
            }

            ExecuteTheRecipe.showlog("工序:手动投料," + name);
            String title = "手动投料-温馨提示！";
            String message = "客官请投入主料,[" + name + "],投入后点击[确定]继续流程！";

            final boolean[] IsComplete = {false};
            Activity activity = findActivity(context);
            if (activity != null) {
                activity.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            new QMUIDialog.MessageDialogBuilder(context).setCancelable(false).setTitle(title).setMessage(message).addAction("确定", new QMUIDialogAction.ActionListener() {
                                @Override
                                public void onClick(QMUIDialog dialog, int index) {
                                    IsComplete[0] = true;
                                    dialog.dismiss();
                                }
                            }).create(com.qmuiteam.qmui.R.style.QMUI_Dialog).show();
                        } catch (Exception e) {

                        }
                    }
                });
            }

            while (!IsComplete[0]) {
                Thread.sleep(100);//10 *6
            }

            ExecuteTheRecipe.showlog("手动已确认投入主料:" + name + "-继续流程");
        } catch (Exception ex) {
            ToastUtils.error("异常信息:" + ex.getMessage());
        }
    }

    /**
     * 写PLC搅拌
     *
     * @param data
     */
    private static void Write_PLC_Stir(HashMap<String, String> data) {
        try {
            int seep = 0;
            boolean status = false;
            for (HashMap.Entry<String, String> entry : data.entrySet()) {
                String key = entry.getKey();
                String value = entry.getValue();
                if (key.contains("搅拌速度")) {
                    seep = Integer.parseInt(value);
                } else if (key.contains("搅拌状态")) {
                    status = value.contains("启动") ? true : false;
                }
            }

            WritePLC("搅拌速度", seep);

            if (status) {
                BottomClick("搅拌启动开关");
            } else {
                BottomClick("搅拌停止开关");
            }
        } catch (Exception ex) {
            ToastUtils.error("异常信息:" + ex.getMessage());
        }
    }

    /**
     * 写PLC 设置加热功率
     *
     * @param data
     */
    private static void Write_PLC_Heating(HashMap<String, String> data) {
        try {
            int val1 = 0;
            int val2 = 0;
            int val3 = 0;
            int val4 = 0;

            for (HashMap.Entry<String, String> entry : data.entrySet()) {
                String key = entry.getKey();
                String value = entry.getValue();
                if (key.contains("一环功率")) {
                    val1 = Integer.parseInt(value);
                } else if (key.contains("二环功率")) {
                    val2 = Integer.parseInt(value);
                } else if (key.contains("三环功率")) {
                    val3 = Integer.parseInt(value);
                } else if (key.contains("四环功率")) {
                    val4 = Integer.parseInt(value);
                }
            }

            WritePLC("小火开关（一圈）", val1);
            WritePLC("中火开关（二圈）", val2);
            WritePLC("大火开关（三圈）", val3);
            WritePLC("强火开关（四圈）", val4);
        } catch (Exception ex) {
            ToastUtils.error("异常信息:" + ex.getMessage());
        }
    }

    /**
     * 写PLC 约束条件
     *
     * @param data
     */
    private static void Write_PLC_ConstraintCondition(HashMap<String, String> data) {
        try {
            int time = 0;
            int temp = 0;
            int val = 0;
            for (HashMap.Entry<String, String> entry : data.entrySet()) {
                String key = entry.getKey();
                String value = entry.getValue();
                if (key.contains("(秒)")) {
                    time = Integer.parseInt(value);
                }
                if (key.contains("锅内重量")) {
                    val = Integer.parseInt(value);
                }
                if (key.contains("温度设定")) {
                    temp = Integer.parseInt(value);
                }
            }


            //1.判断等待时长是否满足
            if (time > 0 && !IsForcedEnd) {
                ThreadDelay(time);
            }

            //2.判断温度是否满足
            if (temp > 0 && !IsForcedEnd) {
                ThreadWhile("炒锅当前温度", temp);
            }

            //3.判断重量是否到达
            if (val > 0 && !IsForcedEnd) {
                ThreadWhile("炒锅当前重量", val);
            }

        } catch (Exception ex) {
            ToastUtils.error("异常信息:" + ex.getMessage());
        }
    }
    //endregion

    //region PLC监听线程

    /**
     * 自提设备监听
     */
    public static void Listening() {

        ConfigName.getInstance().PLC_Address.clear();
        ArrayList<BPA_PLCADDRESS> address = QueryDB.GetPlcaddressALL();
        for (BPA_PLCADDRESS item : address) {
            if (!item.address.isEmpty() && !item.address.equals("-------------")) {
                ConfigName.getInstance().PLC_Address.put(item.name, item);
            }
        }

        new Thread(new Runnable() {
            @Override
            public void run() {
                WritePLC("系统启停开关", true);
            }
        }).start();

        ThreadManager.Get().StartLong("PLC设备数据监听", true, new IThread() {
            @Override
            public void Run() throws InterruptedException {
                try {
                    for (String item : ConfigName.getInstance().PLC_Address.keySet()) {
                        String key = item;
                        BPA_PLCADDRESS plcaddress = ConfigName.getInstance().PLC_Address.get(item);
                        if (plcaddress.isread == 1) {
                            Object val = ReadPLC(key);
                            ListeningValue.put(key, val);
                        }
                    }

                } catch (Exception ex) {
                    ToastUtils.error("异常信息:" + ex.getMessage());
                }
                Thread.sleep(100);
            }

            @Override
            public void RunComplete() throws InterruptedException {

            }
        });

        ThreadManager.Get().StartLong("PLC设备-心跳", true, new IThread() {
            @Override
            public void Run() throws InterruptedException {
                try {
                    if (ConfigName.getInstance().PlcIsConnect) {
                        //心跳
                        WritePLC("上位机-PLC", true);
                        Thread.sleep(500);
                        WritePLC("上位机-PLC", false);
                    }
                } catch (Exception ex) {
                    ToastUtils.error("异常信息:" + ex.getMessage());
                }
                Thread.sleep(500);
            }

            @Override
            public void RunComplete() throws InterruptedException {

            }
        });
    }
    //endregion

    //region 界面事件
    /**
     * 阻断按钮
     *
     * @param name
     */
    public static void BottomClick(String name) {
        try {
            Log.e("按钮按下", name);
            WritePLC(name, true);
            Thread.sleep(200);
            WritePLC(name, false);
            Log.e("按钮复位", name);
            ExecuteTheRecipe.showlog("写入plc-------" + name);
        } catch (Exception ex) {
            Log.e("按钮异常", name + ex.getMessage());
            WritePLC(name, false);
        }
    }

    /**
     * 发现主窗体
     * @param context
     * @return
     */
    private static Activity findActivity(@NonNull Context context) {
        if (context instanceof Activity) {
            return (Activity) context;
        } else if (context instanceof ContextWrapper) {
            return findActivity(((ContextWrapper) context).getBaseContext());
        } else {
            return null;
        }
    }
    //endregion

    //region 延迟函数
    /**
     * 线程延迟函数(避免线程阻塞等待)
     *
     * @param delay
     */
    public static void ThreadDelay(int delay) {
        ExecuteTheRecipe.showlog("工序线程等待-------时长：" + delay + "秒,请耐心等待，开始计时！");

        long startTime = System.currentTimeMillis();
        while ((System.currentTimeMillis() - startTime) <= (delay * 1000) && !IsForcedEnd)
        {
            try {
                Thread.sleep(100);
            } catch (InterruptedException ex) {
            }
        }

        long endTime = System.currentTimeMillis(); //结束时间
        int time = (int) ((endTime - startTime) / 1000);
        ExecuteTheRecipe.showlog("工序线程等待结束-------总耗时：" + time);
    }

    /**
     * 线程等待true信号
     *
     * @param name
     */
    public static void ThreadWhile(String name) {
        //"炒菜位1反馈";
        final boolean[] IsComplete = {false};
        long a = System.currentTimeMillis();
        ExecuteTheRecipe.showlog(name + ",等待中");
        while (!IsComplete[0] && !IsForcedEnd) {
            if ((System.currentTimeMillis() - a) > 1000 * whileTime) {
                ExecuteTheRecipe.showlog(name + ",异常超时退出!");
                break;
            } else {
                Object sb = ReadPLC(name);
                if (sb != null) {
                    IsComplete[0] = (boolean) (sb);
                    if(IsComplete[0])
                    {
                        ExecuteTheRecipe.showlog(name + ",接收到信号!");
                    }
                }
            }
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                IsComplete[0] = true;
                ExecuteTheRecipe.showlog(name + ",异常退出!" + e.getMessage());
            }
        }
        ExecuteTheRecipe.showlog(name + ",等待结束!");
    }

    /**
     * 线程等待false信号
     *
     * @param name
     */
    public static void ThreadWhileFlase(String name) {
        //"炒菜位1反馈";
        final boolean[] IsComplete = {true};
        long a = System.currentTimeMillis();
        ExecuteTheRecipe.showlog(name + ",等待中");
        while (IsComplete[0] && !IsForcedEnd) {
            if ((System.currentTimeMillis() - a) > 1000 * whileTime) {
                ExecuteTheRecipe.showlog(name + ",异常超时退出!");
                break;
            } else {
                Object sb = ReadPLC(name);
                if (sb != null) {
                    IsComplete[0] = (boolean) (sb);
                    if(IsComplete[0]==false)
                    {
                        ExecuteTheRecipe.showlog(name + ",接收到信号!");
                    }
                }
            }
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                IsComplete[0] = false;
                ExecuteTheRecipe.showlog(name + ",异常退出!" + e.getMessage());
            }
        }
        ExecuteTheRecipe.showlog(name + ",等待结束!");
    }

    /**
     * 线程等待--等待int数据是否大于int
     *
     * @param name
     */
    public static void ThreadWhile(String name, int val) {
        final boolean[] IsComplete = {false};
        long a = System.currentTimeMillis();
        ExecuteTheRecipe.showlog(name + ",等待数据大于" + val);
        while (!IsComplete[0] && !IsForcedEnd) {
            if ((System.currentTimeMillis() - a) > 1000 * whileTime) {
                ExecuteTheRecipe.showlog(name + ",异常超时退出!");
                break;
            } else {
                Object sb = ReadPLC(name);
                int value = sb == null ? 0 : (int) (sb);
                if (value >= val) {
                    IsComplete[0] = true;
                    ExecuteTheRecipe.showlog(name + ",接收到信号!");
                } else {
                    IsComplete[0] = false;
                }
            }
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                IsComplete[0] = true;
                ExecuteTheRecipe.showlog(name + ",异常退出!" + e.getMessage());
            }
        }
        ExecuteTheRecipe.showlog(name + ",等待结束!");
    }
    //endregion

    //region 获取默认工艺

    /**
     * 获取搅拌
     *
     * @param sd 0-50
     * @return
     */
    public static BPA_GOODSRECIPE Get搅拌(int sd) {
        BPA_GOODSRECIPE pf = new BPA_GOODSRECIPE();
        pf.materialType = 1;
        pf.processname = "搅拌";
        pf.processms = "搅拌(" + sd + ",0)";
        pf.processvalue = "搅拌速度," + sd + "|等待(秒),0";
        return pf;
    }

    /**
     * 获取搅拌
     *
     * @param data 自动打火、手动打火
     * @return
     */
    public static BPA_GOODSRECIPE Get打火(String data) {
        BPA_GOODSRECIPE pf = new BPA_GOODSRECIPE();
        pf.materialType = 1;
        pf.processname = "打火";
        pf.processms = "打火(" + data + ",0)";
        pf.processvalue = "打火动作," + data + "|等待(秒),0";
        return pf;
    }

    /**
     * 获取位置
     *
     * @param
     * @return
     */
    public static BPA_GOODSRECIPE Get位置(String name) {
        BPA_GOODSRECIPE pf = new BPA_GOODSRECIPE();
        pf.materialType = 1;
        pf.processname = "位置";
        pf.processms = "位置(" + name + ",0)";
        pf.processvalue = "位置动作," + name + "|等待(秒),0";
        return pf;
    }

    /**
     * 获取手动主料
     *
     * @param
     * @return
     */
    public static BPA_GOODSRECIPE Get主料(int k1, int k2, int k3, int k4, int temp, int zl) {
        BPA_GOODSRECIPE pf = new BPA_GOODSRECIPE();
        pf.materialType = 1;
        pf.processname = "主料";
        pf.processms = String.format("主料(未知,%d,%d,%d,%d,%d,%d,%d", k1, k2, k3, k4, temp, zl, 0);
        pf.processvalue = String.format("主料名称,未知|一环功率,%d|二环功率,%d|三环功率,%d|四环功率,%d|温度设定,%d|锅内重量,%d|烹饪(秒),%d", k1, k2, k3, k4, temp, zl, 0);
        return pf;
    }

    /**
     * 获取约束条件
     *
     * @param
     * @return
     */
    public static BPA_GOODSRECIPE Get约束条件(int k1, int k2, int k3, int k4, int temp, int zl) {
        BPA_GOODSRECIPE pf = new BPA_GOODSRECIPE();
        pf.materialType = 1;
        pf.processname = "约束条件";
        pf.processms = String.format("约束条件(%d,%d,%d,%d,%d,%d,%d", k1, k2, k3, k4, temp, zl, 0);
        pf.processvalue = String.format("一环功率,%d|二环功率,%d|三环功率,%d|四环功率,%d|温度设定,%d|锅内重量,%d|烹饪(秒),%d", k1, k2, k3, k4, temp, zl, 0);
        return pf;
    }

    /**
     * 获取选中行的变量
     *
     * @return
     */
    public static BPA_GOODSRECIPE Get物料(String name, String yl) {
        BPA_GOODSRECIPE pf = new BPA_GOODSRECIPE();
        try {
            //  延迟,100|延迟,100|延迟,100|延迟,100|
            String data = "";
            String desc = "";

            pf.materialType = 0;
            data += name + "," + yl + "|";
            desc += yl + ",";

            pf.processname = "物料";
            pf.processms = "物料" + "(" + desc.substring(0, desc.length() - 1) + ")";
            pf.processvalue = data.substring(0, data.length() - 1);
            return pf;
        } catch (Exception ex) {
            return null;
        }
    }

    /**
     * 获取延迟
     *
     * @param time
     * @return
     */
    public static BPA_GOODSRECIPE Get延迟(int time) {
        BPA_GOODSRECIPE pf = new BPA_GOODSRECIPE();
        pf.materialType = 1;
        pf.processname = "延迟";
        pf.processms = "延迟(" + time + ")";
        pf.processvalue = "等待(秒)," + time;
        return pf;
    }

    /**
     * 获取出菜
     *
     * @return
     */
    public static BPA_GOODSRECIPE Get出菜(int k) {
        BPA_GOODSRECIPE pf = new BPA_GOODSRECIPE();
        pf.materialType = 1;
        pf.processname = "出菜";
        pf.processms = "出菜(" + k + ")";
        pf.processvalue = "等待(秒)," + k;
        return pf;
    }

    /**
     * 获取清洗
     *
     * @return
     */
    public static BPA_GOODSRECIPE Get清洗(int k) {
        BPA_GOODSRECIPE pf = new BPA_GOODSRECIPE();
        pf.materialType = 1;
        pf.processname = "清洗";
        pf.processms = "清洗(" + k + ")";
        pf.processvalue = "等待(秒)," + k;
        return pf;
    }
    //endregion

    //region 显示日志

    /**
     * 显示日志
     */
    public static void showlog(String msg) {
        Log.e("hbl", msg);
        ToastUtils.info(msg);
    }
    //endregion

    //region PLC控制类

    /**
     * 写PLC数据
     *
     * @param name
     * @param value
     */
    public static void WritePLC(String name, Object value) {
        try {
            if (ConfigName.getInstance().PLC_Address.containsKey(name)) {
                BPA_PLCADDRESS plcaddress = ConfigName.getInstance().PLC_Address.get(name);
                if (!plcaddress.address.isEmpty() && ConfigName.getInstance().PlcIsConnect) {
                    SiemensHelper1.get().writePLC(plcaddress.address, value);
                }
            }
        } catch (Exception ex) {
            ToastUtils.error("异常信息:" + ex.getMessage());
        } finally {
        }
    }

    /**
     * 读PLC数据
     *
     * @param name
     * @return
     */
    public static Object ReadPLC(String name) {
        final Object[] ReturnsVariable = {null};
        try {
            if (ConfigName.getInstance().PLC_Address.containsKey(name)) {
                BPA_PLCADDRESS plcaddress = ConfigName.getInstance().PLC_Address.get(name);
                if (!plcaddress.address.isEmpty() && ConfigName.getInstance().PlcIsConnect) {

                    ReturnsVariable[0] = SiemensHelper1.get().readPLC(plcaddress.address);
                }
            }
        } catch (Exception ex) {
            ToastUtils.error("异常信息:" + ex.getMessage());
        } finally {
            return ReturnsVariable[0];
        }
    }

    /**
     * 获取监听变量
     *
     * @param
     * @return
     */
    public static Object getListingValue(String name) {
        if (!ListeningValue.containsKey(name)) {
            return null;
        }
        return ListeningValue.get(name);
    }
    //endregion
}
