package com.bonait.bnframework.common.modbus.s7;

import static com.github.xingshuangs.iot.common.constant.GeneralConst.S7_PORT;

import android.util.Log;

import com.github.xingshuangs.iot.exceptions.SocketRuntimeException;
import com.github.xingshuangs.iot.protocol.s7.enums.EPlcType;
import com.github.xingshuangs.iot.protocol.s7.model.DataItem;
import com.github.xingshuangs.iot.protocol.s7.service.S7PLC;
import com.github.xingshuangs.iot.protocol.s7.utils.AddressUtil;
import com.github.xingshuangs.iot.utils.FloatUtil;
import com.github.xingshuangs.iot.utils.IntegerUtil;
import com.github.xingshuangs.iot.utils.ShortUtil;

import java.lang.reflect.ParameterizedType;
import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SiemensHelper extends S7PLC {

    /**
     * 判断PLC连接是否成功
     * @return
     */
    public boolean IsConnected(){
        try {
            socket.sendUrgentData(0xff);
            return true;
        }catch(Exception e){
            return false;
        }
    }

    public void Delay(long millis){
        try{
            Thread.sleep(millis);
        }catch(InterruptedException e){

        }
    }

    public boolean ReadBoolean(String address){
        try {
            return  this.readBoolean(address);
        }
        catch(SocketRuntimeException ex){
            this.close();
        }
        return false;
    }

    public List<Boolean> ReadBoolean(String... address) {
        try {
            return this.readBoolean(Arrays.asList(address));
        }
        catch(SocketRuntimeException ex){
            this.close();
        }
        return new ArrayList<Boolean>();
    }

    public short ReadInt16(String address) {
        try {
            DataItem dataItem = this.readS7Data(AddressUtil.parseByte(address, 2));
            return ShortUtil.toInt16(dataItem.getData());
        }
        catch(SocketRuntimeException ex){
            this.close();
        }
        return 0;

    }

    public List<Short> ReadInt16(String... address) {
        try {
            return this.readInt16(Arrays.asList(address));
        }
        catch(SocketRuntimeException ex){
            this.close();
        }
        return new ArrayList<Short>();
    }

    public int ReadUInt16(String address) {
        try {
            DataItem dataItem = this.readS7Data(AddressUtil.parseByte(address, 2));
            return ShortUtil.toUInt16(dataItem.getData());
        }
        catch(SocketRuntimeException ex){
            this.close();
        }
        return 0;
    }

    public List<Integer> ReadUInt16(String... address) {
        try {
            return this.readUInt16(Arrays.asList(address));
        }
        catch(SocketRuntimeException ex){
            this.close();
        }
        return new ArrayList<Integer>();
    }

    public int ReadInt32(String address) {
        try {
            DataItem dataItem = this.readS7Data(AddressUtil.parseByte(address, 4));
            return IntegerUtil.toInt32(dataItem.getData());
        }
        catch(SocketRuntimeException ex){
            this.close();
        }
        return 0;
    }

    public List<Integer> ReadInt32(String... address) {
        try {
            return this.readInt32(Arrays.asList(address));
        }
        catch(SocketRuntimeException ex){
            this.close();
        }
        return new ArrayList<Integer>();
    }

    public float ReadFloat32(String address) {
        try {
            DataItem dataItem = this.readS7Data(AddressUtil.parseByte(address, 4));
            return FloatUtil.toFloat32(dataItem.getData());
        }
        catch(SocketRuntimeException ex){
            this.close();
        }
        return 0;
    }

    public List<Float> ReadFloat32(String... address) {
        try {
            return this.readFloat32(Arrays.asList(address));
        }
        catch(SocketRuntimeException ex){
            this.close();
        }
        return new ArrayList<Float>();
    }

    public void WriteBoolean(String address, boolean data) {
        while (true) {
            try {
                this.writeS7Data(AddressUtil.parseBit(address), DataItem.createReqByBoolean(data));
                break;
            }
            catch(SocketRuntimeException ex){
                this.close();
                Delay(100);
            }
        }
    }

    public void WriteUInt16(String address, int data) {
        while (true) {
            try {
                this.writeByte(address, ShortUtil.toByteArray(data));
                break;
            }
            catch(SocketRuntimeException ex){
                this.close();
                Delay(100);
            }
        }
    }

    public void WriteInt16(String address, short data) {
        while (true) {
            try {
                Log.e("info","准备写入数据："+data);
                this.writeByte(address, ShortUtil.toByteArray(data));
                break;
            }
            catch(SocketRuntimeException ex){
                Log.e("socketException","写入："+data+"失败");
                this.close();
                Delay(100);
            }
        }
    }

    public void WriteUInt32(String address, long data) {
        while (true) {
            try {
                this.writeByte(address, IntegerUtil.toByteArray(data));
                break;
            }
            catch(SocketRuntimeException ex){
                this.close();
                Delay(100);
            }
        }
    }

    public void WriteInt32(String address, int data) {
        while (true) {
            try {
                this.writeByte(address, IntegerUtil.toByteArray(data));
                break;
            }
            catch(SocketRuntimeException ex){
                this.close();
                Delay(100);
            }
        }
    }

    public void WriteFloat32(String address, float data) {
        while (true) {
            try {
                this.writeByte(address, FloatUtil.toByteArray(data));
                break;
            }
            catch(SocketRuntimeException ex){
                this.close();
                Delay(100);
            }
        }
    }

    public SiemensHelper(EPlcType PlcType, String ip){
        super(PlcType, ip);
    }

}
