package com.bonait.bnframework.common.modbus.s7;

import android.util.Log;

import com.bonait.bnframework.common.constant.ConfigName;
import com.github.xingshuangs.iot.exceptions.SocketRuntimeException;
import com.github.xingshuangs.iot.protocol.s7.enums.EPlcType;
import com.github.xingshuangs.iot.protocol.s7.service.S7PLC;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;

/**
 * S7
 */
public class SiemensHelper1 extends S7PLC {

    //region 单例
    private static volatile SiemensHelper1 instance = null;

    public static SiemensHelper1 get() {
        SiemensHelper1 manager = instance;
        if (manager == null) {
            synchronized (SiemensHelper1.class) {
                manager = instance;
                if (manager == null) {
                    manager = new SiemensHelper1();
                    instance = manager;
                }
            }
        }
        return manager;
    }

    private SiemensHelper1() {

    }
    //endregion

    //region 变量
    /**
     * 默认PLC类型
     */
    public static EPlcType type = EPlcType.S1500;
    public static S7PLC s7PLC = null;
    //endregion

    //region 公共方法
    /**
     * Ping PLC地址是否通畅
     * @param address
     * @param pingTimes  ping的次数
     * @param timeOut  超时时间 10
     * @return
     */
    public static boolean ping2(String address, int pingTimes, int timeOut) {
        Process process = null;
        try {
            process = Runtime.getRuntime().exec( "ping "  + "-c " + pingTimes + " -w " + timeOut+ " "+address);
            InputStreamReader r = new InputStreamReader(process.getInputStream());

            LineNumberReader returnData = new LineNumberReader(r);

            String returnMsg="";

            String line = "";

            while ((line = returnData.readLine()) != null) {

                System.out.println(line);

                returnMsg += line;

            }

            if(returnMsg.indexOf("100% packet loss")!=-1){
                System.out.println("与 " +address +" 连接不畅通.");
                //ToastUtils.info("与 " +address +" 连接不畅通.");
                return  false;
            } else{
                System.out.println("与 " +address +" 连接畅通.");
                //ToastUtils.info("与 " +address +" 连接不畅通.");
                return true;
            }
        } catch (IOException e) {
            // e.printStackTrace();
        }
        return false;

    }
    /**
     * 连接plc
     */
    public static void Connect() {
        try {
            s7PLC = new S7PLC(type, ConfigName.getInstance().Address);
            s7PLC.connect();
            ConfigName.getInstance().PlcIsConnect = true;
        } catch (SocketRuntimeException ex) {
            Log.e("SocketRuntimeException", ex.getMessage());
            Close();
        } catch (Exception ex) {
            Log.e("Exception", ex.getMessage());
            Close();
        }

    }

    /**
     * 关闭
     */
    public static void Close() {
        try {
            if (s7PLC != null) {
                s7PLC.close();
            }
        } catch (SocketRuntimeException ex) {
            Log.e("SocketRuntimeException", ex.getMessage());
        } catch (Exception ex) {
            Log.e("Exception", ex.getMessage());
        }
        ConfigName.getInstance().PlcIsConnect = false;
    }

    /**
     * 读取数据
     * @param address
     * @return
     */
    public Object readPLC(String address) {
        Object res=null;
        try {
            if (address.contains("V")) {
                res = s7PLC.readBoolean(address);
            } else if (address.contains("D")) {
                res = s7PLC.readUInt32(address);
            }
        } catch (SocketRuntimeException ex) {
            Close();
        } catch (Exception ex) {

        }
        return res;
    }

    /**
     * 写数据
     * @param address
     * @return
     */
    public boolean writePLC(String address,Object data) {
        boolean res=false;
        try {
            if (address.contains("V")) {
                s7PLC.writeBoolean(address,(boolean)data);
            } else if (address.contains("D")) {
                s7PLC.writeInt32(address,(int)data);
            }
            res=true;
        } catch (SocketRuntimeException ex) {
            Close();
        } catch (Exception ex) {

        }
        return res;
    }
    //endregion
}
